#include <stdio.h>
#include <stdlib.h>
#include <math.h>

struct point {
    double x;
    double y;
};

double dist2origin(const struct point *p) {
    return sqrt(p->x * p->x + p->y * p->y);
}

int compare_points(const void *p1, const void *p2) {
    struct point *q1 = (struct point*)p1;
    struct point *q2 = (struct point*)p2;
    double d1 = dist2origin(q1);
    double d2 = dist2origin(q2);
    if (d1 != d2) {
        return d1 < d2 ? -1 : 1;
    } else {
        double a1 = atan2(q1->y, q1->x);
        double a2 = atan2(q2->y, q2->x);
        return a1 == a2 ? 0 : (a1 < a2 ? -1 : 1);
    }
}

void sort_points(struct point *points,
                 unsigned int n) {
    qsort(points, n, sizeof(struct point), compare_points);
}

void print_points(const struct point *points,
                  unsigned int n) {
    for (unsigned int i = 0; i < n; ++i) {
        printf("(%lf, %lf) ", points[i].x, points[i].y);
    }
}

int main(void) {
    struct point points[] = {
        {1.0, 0.0}, {1.0, 1.0}, {-1.0, 1.0}, {0.5, 0.5}
    };
    sort_points(points, 4);
    print_points(points, 4);
    return 0;
}
